
/*
 * Kana test
 *
 * Copyright (C) 2001 Tomasz Mka
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include <time.h>
#include <string.h>

#include "gui.h"
#include "main.h"
#include "prefs.h"
#include "test.h"

#include "icon.h"
#include "logo.h"


static GtkWidget *main_window = NULL;
static GtkWidget *stat_window = NULL;
static GdkPixmap *pixmap;
static GdkPixmap *pixmap_m;
static GtkWidget *logo;
static GdkBitmap *mask;
static GtkWidget *char_label;
static GtkWidget *start_button;
static GtkWidget *quit_button;
static GtkWidget *hiragana_radio;
static GtkWidget *katakana_radio;
static GtkWidget *mixed_radio;

gint stop_flag = FALSE;

GtkWidget *rm_entry;
GtkWidget *stop_button;

time_t timer_int_l;

/*--------------------------------------------------------------------*/

void gui_set_progress (void)
{
gchar buffer[32];

	sprintf (buffer, "[ %d / %d ]", q_counter+1, max_entries);
	gtk_label_set_text (GTK_LABEL(char_label), buffer);
}

/*--------------------------------------------------------------------*/

void stop_func ( GtkWidget *widget, gpointer data )
{
	gui_disable_test ();
}

/*--------------------------------------------------------------------*/

void start_func ( GtkWidget *widget, gpointer data )
{

	q_counter = a_counter = 0;

	if (test_mode == MIXED)
		max_entries = MAX_SIGNS;
	else
		max_entries = N_SIGNS;

	gui_disable_start ();
	test_gen_hashtab (max_entries);
	gui_set_progress ();

	gui_put_sign (qst_hash[q_counter], test_mode);

	timer_int_l = time (NULL);
}

/*--------------------------------------------------------------------*/

void close_window( GtkWidget *widget, gpointer data )
{
	gdk_window_get_root_origin (main_window->window,
								&config.window_x, &config.window_y);

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(hiragana_radio)) == TRUE)
		config.last_mode = HIRAGANA;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(katakana_radio)) == TRUE)
		config.last_mode = KATAKANA;
	else
		config.last_mode = MIXED;

	gtk_main_quit ();
}

/*--------------------------------------------------------------------*/

void rm_key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
{

	switch (event->keyval) {

		case GDK_Escape:
			gtk_entry_set_text (GTK_ENTRY(rm_entry), "");
			break;

		case GDK_Return:
            if (strlen (gtk_entry_get_text (GTK_ENTRY(rm_entry))))
					test_main (gtk_entry_get_text (GTK_ENTRY(rm_entry)));
			break;

	}
}

/*--------------------------------------------------------------------*/

void testmode_select (GtkWidget *entry, GtkUpdateType type)
{
	test_mode = type;
}

/*--------------------------------------------------------------------*/

void gui_create_window (void)
{
GtkWidget *frame1;
GtkWidget *vbox1;
GtkWidget *frame2;
GtkWidget *hseparator1;
GtkWidget *hseparator2;
GtkWidget *frame3;
GtkWidget *frame4;
GtkWidget *hbox1;
GSList *hbox1_group = NULL;
GtkWidget *hbuttonbox1;
GdkBitmap *mask;
GdkPixmap *icon;

	main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy (GTK_WINDOW (main_window), FALSE, TRUE, FALSE);
	gtk_window_set_title (GTK_WINDOW (main_window), "Kana test " VERSION);
	gtk_widget_set_usize (main_window, 242, 338);

	gtk_widget_set_uposition (GTK_WIDGET (main_window),
								config.window_x, config.window_y);

	gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
						GTK_SIGNAL_FUNC(close_window), NULL);

	gtk_widget_show (main_window);

	frame1 = gtk_frame_new (NULL);
	gtk_widget_show (frame1);
	gtk_container_add (GTK_CONTAINER (main_window), frame1);
	gtk_container_set_border_width (GTK_CONTAINER (frame1), 2);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (frame1), vbox1);

	frame2 = gtk_frame_new (NULL);
	gtk_widget_show (frame2);
	gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame2), 28);
	gtk_frame_set_shadow_type (GTK_FRAME (frame2), GTK_SHADOW_ETCHED_OUT);

	pixmap_m = gdk_pixmap_create_from_xpm_d(main_window->window, &mask, NULL, logo_xpm);
                                           
	logo = gtk_pixmap_new (pixmap_m, mask);
	gtk_widget_show (logo);
	gtk_container_add (GTK_CONTAINER (frame2), logo);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, TRUE, 0);

	char_label = gtk_label_new ("");
	gtk_widget_show (char_label);
	gtk_box_pack_start (GTK_BOX (vbox1), char_label, FALSE, FALSE, 0);
	gtk_misc_set_alignment (GTK_MISC (char_label), 0.5, 0.5);
	gtk_misc_set_padding (GTK_MISC (char_label), 0, 5);

	hseparator2 = gtk_hseparator_new ();
	gtk_widget_show (hseparator2);
	gtk_box_pack_start (GTK_BOX (vbox1), hseparator2, FALSE, TRUE, 0);

	frame3 = gtk_frame_new ("Romaji: ");
	gtk_widget_show (frame3);
	gtk_box_pack_start (GTK_BOX (vbox1), frame3, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame3), 3);

	rm_entry = gtk_entry_new ();
	gtk_widget_show (rm_entry);
	gtk_signal_connect (GTK_OBJECT(rm_entry), "key_press_event",
						GTK_SIGNAL_FUNC(rm_key_press), NULL);
	gtk_container_add (GTK_CONTAINER (frame3), rm_entry);

	frame4 = gtk_frame_new ("Test mode:");
	gtk_widget_show (frame4);
	gtk_box_pack_start (GTK_BOX (vbox1), frame4, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame4), 3);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_container_add (GTK_CONTAINER (frame4), hbox1);

	hiragana_radio = gtk_radio_button_new_with_label (hbox1_group, "Hiragana");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (hiragana_radio));
	gtk_widget_show (hiragana_radio);
	gtk_signal_connect (GTK_OBJECT (hiragana_radio), "clicked",
						GTK_SIGNAL_FUNC (testmode_select),
					    GINT_TO_POINTER (HIRAGANA));
	gtk_box_pack_start (GTK_BOX (hbox1), hiragana_radio, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hiragana_radio), 4);

	katakana_radio = gtk_radio_button_new_with_label (hbox1_group, "Katakana");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (katakana_radio));
	gtk_widget_show (katakana_radio);
	gtk_signal_connect (GTK_OBJECT (katakana_radio), "clicked",
						GTK_SIGNAL_FUNC (testmode_select),
					    GINT_TO_POINTER (KATAKANA));
	gtk_box_pack_start (GTK_BOX (hbox1), katakana_radio, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (katakana_radio), 4);

	mixed_radio = gtk_radio_button_new_with_label (hbox1_group, "Mixed");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (mixed_radio));
	gtk_widget_show (mixed_radio);
	gtk_signal_connect (GTK_OBJECT (mixed_radio), "clicked",
						GTK_SIGNAL_FUNC (testmode_select),
					    GINT_TO_POINTER (MIXED));
	gtk_box_pack_start (GTK_BOX (hbox1), mixed_radio, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (mixed_radio), 4);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 5);
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 75, -1);

	start_button = gtk_button_new_with_label ("Start");
	gtk_signal_connect (GTK_OBJECT (start_button), "clicked",
						GTK_SIGNAL_FUNC (start_func), NULL);
	gtk_widget_show (start_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), start_button);
	GTK_WIDGET_SET_FLAGS (start_button, GTK_CAN_DEFAULT);

	stop_button = gtk_button_new_with_label ("Stop");
	gtk_signal_connect (GTK_OBJECT (stop_button), "clicked",
						GTK_SIGNAL_FUNC (stop_func), NULL);
	gtk_widget_show (stop_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), stop_button);
	GTK_WIDGET_SET_FLAGS (stop_button, GTK_CAN_DEFAULT);

	quit_button = gtk_button_new_with_label ("Quit");
	gtk_signal_connect (GTK_OBJECT (quit_button), "clicked",
						GTK_SIGNAL_FUNC (close_window), NULL);
	gtk_widget_show (quit_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), quit_button);
	GTK_WIDGET_SET_FLAGS (quit_button, GTK_CAN_DEFAULT);

	icon = gdk_pixmap_create_from_xpm_d (main_window->window,
										&mask, NULL, icon_xpm);
                                           
	gdk_window_set_icon (main_window->window, NULL, icon, icon);

	gui_disable_test ();

	if (test_mode == HIRAGANA)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(hiragana_radio), TRUE);
	else if (test_mode == KATAKANA)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(katakana_radio), TRUE);
	else
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mixed_radio), TRUE);

}

/*--------------------------------------------------------------------*/

void gui_disable_test (void)
{
	gtk_widget_set_sensitive (start_button, TRUE);
	gtk_widget_set_sensitive (quit_button, TRUE);
	gtk_widget_set_sensitive (hiragana_radio, TRUE);
	gtk_widget_set_sensitive (katakana_radio, TRUE);
	gtk_widget_set_sensitive (mixed_radio, TRUE);

	gtk_widget_set_sensitive (rm_entry, FALSE);
	gtk_widget_set_sensitive (stop_button, FALSE);

	gtk_label_set_text (GTK_LABEL(char_label), "Select test mode and press <Start>");
	gtk_entry_set_text (GTK_ENTRY(rm_entry), "");
	gtk_pixmap_set (GTK_PIXMAP(logo), pixmap_m, mask);

	gtk_widget_grab_focus (start_button);
}

/*--------------------------------------------------------------------*/

void gui_disable_start (void)
{
	gtk_widget_set_sensitive (rm_entry, TRUE);
	gtk_widget_set_sensitive (stop_button, TRUE);

	gtk_widget_set_sensitive (start_button, FALSE);
	gtk_widget_set_sensitive (quit_button, FALSE);
	gtk_widget_set_sensitive (hiragana_radio, FALSE);
	gtk_widget_set_sensitive (katakana_radio, FALSE);
	gtk_widget_set_sensitive (mixed_radio, FALSE);

	gtk_widget_grab_focus (rm_entry);
}

/*--------------------------------------------------------------------*/

void gui_put_sign (gint number, gint mode)
{
gchar tmp_file[MAX_PATH];

	switch (mode) {
		case HIRAGANA:
			s_strcpy (tmp_file, hiragana_path, MAX_PATH);
			break;
		case KATAKANA:
			s_strcpy (tmp_file, katakana_path, MAX_PATH);
			break;
		case ROMAJI:
			s_strcpy (tmp_file, romaji_path, MAX_PATH);
			break;
		case MIXED:
			if (number >= N_SIGNS) {
				s_strcpy (tmp_file, katakana_path, MAX_PATH);
			} else {
				s_strcpy (tmp_file, hiragana_path, MAX_PATH);
			}
			break;
		default:
			printf ("WARNING: Invalid mode selected.\n");
			break;
	}

	s_strcat (tmp_file, jsigns[number % N_SIGNS], MAX_PATH);
	s_strcat (tmp_file, ".xpm", MAX_PATH);

	pixmap = gdk_pixmap_create_from_xpm (main_window->window, &mask, NULL, tmp_file);

	if (pixmap)
	    gtk_pixmap_set (GTK_PIXMAP(logo), pixmap, mask);
	else {
		printf ("ERROR: file not found (check installation).\n");
		exit (-1);
	}
}

/*--------------------------------------------------------------------*/

void stat_window_close(void)
{
	gdk_window_get_root_origin (stat_window->window,
								&config.stat_window_x, &config.stat_window_y);

	gtk_widget_set_sensitive (main_window, TRUE);

	gtk_widget_destroy (stat_window);
	stat_window = NULL;
}

/*--------------------------------------------------------------------*/

void gui_show_stat (gint mode)
{
GtkWidget *vbox1;
GtkWidget *frame1;
GtkWidget *vbox2;
GtkWidget *hbox1, *hbox8, *hbox2, *hbox10, *hbox9;
GtkWidget *h_label_1, *hi_label_1;
GtkWidget *h_label_2, *hi_label_2;
GtkWidget *h_label_3, *hi_label_3;
GtkWidget *h_label_4, *hi_label_4;
GtkWidget *h_label_5, *hi_label_5;
GtkWidget *frame2;
GtkWidget *vbox3;
GtkWidget *vbox5;
GtkWidget *hbox11, *hbox12, *hbox13, *hbox14, *hbox15;
GtkWidget *k_label_1, *ki_label_1;
GtkWidget *k_label_2, *ki_label_2;
GtkWidget *k_label_3, *ki_label_3;
GtkWidget *k_label_4, *ki_label_4;
GtkWidget *k_label_5, *ki_label_5;
GtkWidget *frame3;
GtkWidget *vbox4, *vbox6;
GtkWidget *hbox16;
GtkWidget *m_label_1, *mi_label_1;
GtkWidget *hbox17, *hbox18, *hbox19, *hbox20;
GtkWidget *m_label_2, *mi_label_2;
GtkWidget *m_label_3, *mi_label_3;
GtkWidget *m_label_4, *mi_label_4;
GtkWidget *m_label_5, *mi_label_5;
GtkWidget *hseparator1;
GtkWidget *hbuttonbox1;
GtkWidget *close_button;

gchar tmpbuffer1[256], tmpbuffer2[256], tmpbuffer3[256];


	if(stat_window != NULL) {
		gdk_window_raise (stat_window->window);
		return;
	}

	gtk_widget_set_sensitive (main_window, FALSE);

	stat_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy (GTK_WINDOW (stat_window), FALSE, TRUE, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (stat_window), 2);
	gtk_window_set_title (GTK_WINDOW (stat_window), "Statistics");
	gtk_widget_set_usize (stat_window, 320, 380);

	gtk_signal_connect (GTK_OBJECT (stat_window), "delete_event",
						GTK_SIGNAL_FUNC(stat_window_close), NULL);

	gtk_widget_set_uposition (GTK_WIDGET (stat_window),
								config.stat_window_x, config.stat_window_y);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (stat_window), vbox1);

	sprintf (tmpbuffer1, "[ HIRAGANA MODE - %d test(s) ]", config.hiragana_counter);
	frame1 = gtk_frame_new (tmpbuffer1);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame1), 2);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame1), vbox2);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

	h_label_1 = gtk_label_new ("Correct answers:");
	gtk_widget_show (h_label_1);
	gtk_box_pack_start (GTK_BOX (hbox1), h_label_1, FALSE, FALSE, 4);

	hi_label_1 = gtk_label_new ("");
	gtk_widget_show (hi_label_1);
	gtk_box_pack_start (GTK_BOX (hbox1), hi_label_1, FALSE, FALSE, 0);

	hbox8 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox8);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox8, TRUE, TRUE, 0);

	h_label_2 = gtk_label_new ("Incorrect answers:");
	gtk_widget_show (h_label_2);
	gtk_box_pack_start (GTK_BOX (hbox8), h_label_2, FALSE, FALSE, 4);

	hi_label_2 = gtk_label_new ("");
	gtk_widget_show (hi_label_2);
	gtk_box_pack_start (GTK_BOX (hbox8), hi_label_2, FALSE, FALSE, 0);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, TRUE, TRUE, 0);

	h_label_3 = gtk_label_new ("Best result:");
	gtk_widget_show (h_label_3);
	gtk_box_pack_start (GTK_BOX (hbox2), h_label_3, FALSE, FALSE, 4);

	hi_label_3 = gtk_label_new ("");
	gtk_widget_show (hi_label_3);
	gtk_box_pack_start (GTK_BOX (hbox2), hi_label_3, FALSE, FALSE, 0);

	hbox10 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox10);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox10, TRUE, TRUE, 0);

	h_label_4 = gtk_label_new ("This test time:");
	gtk_widget_show (h_label_4);
	gtk_box_pack_start (GTK_BOX (hbox10), h_label_4, FALSE, FALSE, 4);

	hi_label_4 = gtk_label_new ("");
	gtk_widget_show (hi_label_4);
	gtk_box_pack_start (GTK_BOX (hbox10), hi_label_4, FALSE, FALSE, 0);

	hbox9 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox9);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox9, TRUE, TRUE, 0);

	h_label_5 = gtk_label_new ("Overall tests time:");
	gtk_widget_show (h_label_5);
	gtk_box_pack_start (GTK_BOX (hbox9), h_label_5, FALSE, FALSE, 4);

	hi_label_5 = gtk_label_new ("");
	gtk_widget_show (hi_label_5);
	gtk_box_pack_start (GTK_BOX (hbox9), hi_label_5, FALSE, FALSE, 0);

	sprintf(tmpbuffer1, "[ KATAKANA MODE - %d test(s) ]", config.katakana_counter);
	frame2 = gtk_frame_new (tmpbuffer1);
	gtk_widget_show (frame2);
	gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame2), 2);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_container_add (GTK_CONTAINER (frame2), vbox3);

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);
	gtk_box_pack_start (GTK_BOX (vbox3), vbox5, TRUE, TRUE, 0);

	hbox11 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox11);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox11, TRUE, TRUE, 0);

	k_label_1 = gtk_label_new ("Correct answers:");
	gtk_widget_show (k_label_1);
	gtk_box_pack_start (GTK_BOX (hbox11), k_label_1, FALSE, FALSE, 4);

	ki_label_1 = gtk_label_new ("");
	gtk_widget_show (ki_label_1);
	gtk_box_pack_start (GTK_BOX (hbox11), ki_label_1, FALSE, FALSE, 0);

	hbox12 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox12);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox12, TRUE, TRUE, 0);

	k_label_2 = gtk_label_new ("Incorrect answers:");
	gtk_widget_show (k_label_2);
	gtk_box_pack_start (GTK_BOX (hbox12), k_label_2, FALSE, FALSE, 4);

	ki_label_2 = gtk_label_new ("");
	gtk_widget_show (ki_label_2);
	gtk_box_pack_start (GTK_BOX (hbox12), ki_label_2, FALSE, FALSE, 0);

	hbox13 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox13);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox13, TRUE, TRUE, 0);

	k_label_3 = gtk_label_new ("Best result:");
	gtk_widget_show (k_label_3);
	gtk_box_pack_start (GTK_BOX (hbox13), k_label_3, FALSE, FALSE, 4);

	ki_label_3 = gtk_label_new ("");
	gtk_widget_show (ki_label_3);
	gtk_box_pack_start (GTK_BOX (hbox13), ki_label_3, FALSE, FALSE, 0);

	hbox14 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox14);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox14, TRUE, TRUE, 0);

	k_label_4 = gtk_label_new ("This test time:");
	gtk_widget_show (k_label_4);
	gtk_box_pack_start (GTK_BOX (hbox14), k_label_4, FALSE, FALSE, 4);

	ki_label_4 = gtk_label_new ("");
	gtk_widget_show (ki_label_4);
	gtk_box_pack_start (GTK_BOX (hbox14), ki_label_4, FALSE, FALSE, 0);

	hbox15 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox15);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox15, TRUE, TRUE, 0);

	k_label_5 = gtk_label_new ("Overall tests time:");
	gtk_widget_show (k_label_5);
	gtk_box_pack_start (GTK_BOX (hbox15), k_label_5, FALSE, FALSE, 4);

	ki_label_5 = gtk_label_new ("");
	gtk_widget_show (ki_label_5);
	gtk_box_pack_start (GTK_BOX (hbox15), ki_label_5, FALSE, FALSE, 0);

	sprintf(tmpbuffer1, "[ MIXED MODE - %d test(s) ]", config.mixed_counter);
	frame3 = gtk_frame_new (tmpbuffer1);
	gtk_widget_show (frame3);
	gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame3), 2);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox4);
	gtk_container_add (GTK_CONTAINER (frame3), vbox4);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox6);
	gtk_box_pack_start (GTK_BOX (vbox4), vbox6, TRUE, TRUE, 0);

	hbox16 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox16);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox16, TRUE, TRUE, 0);

	m_label_1 = gtk_label_new ("Correct answers:");
	gtk_widget_show (m_label_1);
	gtk_box_pack_start (GTK_BOX (hbox16), m_label_1, FALSE, FALSE, 4);

	mi_label_1 = gtk_label_new ("");
	gtk_widget_show (mi_label_1);
	gtk_box_pack_start (GTK_BOX (hbox16), mi_label_1, FALSE, FALSE, 0);

	hbox17 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox17);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox17, TRUE, TRUE, 0);

	m_label_2 = gtk_label_new ("Incorrect answers:");
	gtk_widget_show (m_label_2);
	gtk_box_pack_start (GTK_BOX (hbox17), m_label_2, FALSE, FALSE, 4);

	mi_label_2 = gtk_label_new ("");
	gtk_widget_show (mi_label_2);
	gtk_box_pack_start (GTK_BOX (hbox17), mi_label_2, FALSE, FALSE, 0);

	hbox18 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox18);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox18, TRUE, TRUE, 0);

	m_label_3 = gtk_label_new ("Best result:");
	gtk_widget_show (m_label_3);
	gtk_box_pack_start (GTK_BOX (hbox18), m_label_3, FALSE, FALSE, 4);

	mi_label_3 = gtk_label_new ("");
	gtk_widget_show (mi_label_3);
	gtk_box_pack_start (GTK_BOX (hbox18), mi_label_3, FALSE, FALSE, 0);

	hbox19 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox19);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox19, TRUE, TRUE, 0);

	m_label_4 = gtk_label_new ("This test time:");
	gtk_widget_show (m_label_4);
	gtk_box_pack_start (GTK_BOX (hbox19), m_label_4, FALSE, FALSE, 4);

	mi_label_4 = gtk_label_new ("");
	gtk_widget_show (mi_label_4);
	gtk_box_pack_start (GTK_BOX (hbox19), mi_label_4, FALSE, FALSE, 0);

	hbox20 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox20);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox20, TRUE, TRUE, 0);

	m_label_5 = gtk_label_new ("Overall tests time:");
	gtk_widget_show (m_label_5);
	gtk_box_pack_start (GTK_BOX (hbox20), m_label_5, FALSE, FALSE, 4);

	mi_label_5 = gtk_label_new ("");
	gtk_widget_show (mi_label_5);
	gtk_box_pack_start (GTK_BOX (hbox20), mi_label_5, FALSE, FALSE, 0);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, TRUE, 4);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 4);

	close_button = gtk_button_new_with_label ("Close");
	gtk_widget_show (close_button);
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
						GTK_SIGNAL_FUNC (stat_window_close), NULL);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), close_button);
	GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (close_button);


	gtk_label_set_text (GTK_LABEL(hi_label_5),
						minutes2str(config.hiragana_time));
	gtk_label_set_text (GTK_LABEL(ki_label_5),
						minutes2str(config.katakana_time));
	gtk_label_set_text (GTK_LABEL(mi_label_5),
						minutes2str(config.mixed_time));

	sprintf(tmpbuffer1, "%d correct answer(s).", config.hiragana_max_result);
	gtk_label_set_text (GTK_LABEL(hi_label_3), tmpbuffer1);
	sprintf(tmpbuffer1, "%d correct answer(s).", config.katakana_max_result);
	gtk_label_set_text (GTK_LABEL(ki_label_3), tmpbuffer1);
	sprintf(tmpbuffer1, "%d correct answer(s).", config.mixed_max_result);
	gtk_label_set_text (GTK_LABEL(mi_label_3), tmpbuffer1);

	sprintf(tmpbuffer1, "%d", q_counter-a_counter);
	sprintf(tmpbuffer2, "%d", a_counter);
	sprintf(tmpbuffer3, "%.2f minutes.", tval);

	gtk_widget_set_sensitive (frame1, FALSE);
	gtk_label_set_text (GTK_LABEL(hi_label_1), "-");
	gtk_label_set_text (GTK_LABEL(hi_label_2), "-");
	gtk_label_set_text (GTK_LABEL(hi_label_4), "-");
	gtk_widget_set_sensitive (frame2, FALSE);
	gtk_label_set_text (GTK_LABEL(ki_label_1), "-");
	gtk_label_set_text (GTK_LABEL(ki_label_2), "-");
	gtk_label_set_text (GTK_LABEL(ki_label_4), "-");
	gtk_widget_set_sensitive (frame3, FALSE);
	gtk_label_set_text (GTK_LABEL(mi_label_1), "-");
	gtk_label_set_text (GTK_LABEL(mi_label_2), "-");
	gtk_label_set_text (GTK_LABEL(mi_label_4), "-");

	switch (mode) {
		case HIRAGANA:
			gtk_widget_set_sensitive (frame1, TRUE);
			gtk_label_set_text (GTK_LABEL(hi_label_1), tmpbuffer1);
			gtk_label_set_text (GTK_LABEL(hi_label_2), tmpbuffer2);
			gtk_label_set_text (GTK_LABEL(hi_label_4), tmpbuffer3);
			break;

		case KATAKANA:
			gtk_widget_set_sensitive (frame2, TRUE);
			gtk_label_set_text (GTK_LABEL(ki_label_1), tmpbuffer1);
			gtk_label_set_text (GTK_LABEL(ki_label_2), tmpbuffer2);
			gtk_label_set_text (GTK_LABEL(ki_label_4), tmpbuffer3);
			break;

		case MIXED:
			gtk_widget_set_sensitive (frame3, TRUE);
			gtk_label_set_text (GTK_LABEL(mi_label_1), tmpbuffer1);
			gtk_label_set_text (GTK_LABEL(mi_label_2), tmpbuffer2);
			gtk_label_set_text (GTK_LABEL(mi_label_4), tmpbuffer3);
			break;

		default:
			printf("WARNING: Invalid mode selected.\n");
			break;
	}

	gtk_widget_show (stat_window);

}

/*--------------------------------------------------------------------*/

