
/*
 * Kana test
 *
 * Copyright (C) 2001 Tomasz Mka
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>

#include <gtk/gtk.h>

#include "gui.h"
#include "prefs.h"
#include "main.h"
#include "test.h"


gchar hiragana_path[MAX_PATH];
gchar katakana_path[MAX_PATH];
gchar romaji_path[MAX_PATH];

gint q_counter;
gint a_counter;
gint max_entries;

gfloat tval;

gint qst_hash[MAX_SIGNS];
gint qst_hash_l[MAX_SIGNS];

gchar *jsigns[] = {	"a",		"i",			"u",			"e",		"o",
					"ka",		"ki",			"ku",			"ke",		"ko",
					"sa",		"shi",			"su",			"se",		"so",
					"ta",		"chi",			"tsu",			"te",		"to",
					"na",		"ni",			"nu",			"ne",		"no",
					"ha",		"hi",			"fu",			"he",		"ho",
					"ma",		"mi",			"mu",			"me",		"mo",
					"ya",						"yu",						"yo",
					"ra",		"ri",			"ru",			"re",		"ro",
					"wa",													"wo",
					"n",

					"ga",		"gi",			"gu",			"ge",		"go",
					"za",		"ji_shi",   	"zu_su",		"ze",		"zo",
					"da",		"ji_chi",		"zu_tsu",		"de",		"do",
					"ba",		"bi",			"bu",			"be",		"bo",
					"pa",		"pi",			"pu",			"pe",		"po",

					"kya",						"kyu",						"kyo",
					"sha",						"shu",						"sho",
					"cha",						"chu",						"cho",
					"nya",						"nyu",						"nyo",
					"hya",						"hyu",						"hyo",
					"mya",						"myu",						"myo",
					"rya",						"ryu",						"ryo",

					"gya",						"gyu",						"gyo",
					"ja",						"ju",						"jo",
					"bya",						"byu",						"byo",
					"pya",						"pyu",						"pyo"
};

/*--------------------------------------------------------------------*/

void test_gen_hashtab (gint number)
{
gint i, tmp;
gint zm, zm_s, zm_d;

	srandom (time (NULL) + getpid ());

	for (i = 0; i < number; i++) {
		qst_hash[i] = i;
		qst_hash_l[i] = 1;
	}
    
	zm = (gint)((gfloat)number * 16.0 * random() / (RAND_MAX + (float)number));
    
    
	for (i = 0; i < zm; i++) {
		do {
			zm_s = (gint)((gfloat)number * random() / RAND_MAX);
			zm_d = (gint)((gfloat)number * random() / RAND_MAX);
		} while (zm_s == zm_d);
        
		tmp = qst_hash[zm_s];
		qst_hash[zm_s] = qst_hash[zm_d];
		qst_hash[zm_d] = tmp;
	}
	
}

/*--------------------------------------------------------------------*/

void test_init (void)
{
	s_strcpy (hiragana_path, config.data_path, MAX_PATH);
	s_strcat (hiragana_path, hiragana_dir, MAX_PATH);

	s_strcpy (katakana_path, config.data_path, MAX_PATH);
	s_strcat (katakana_path, katakana_dir, MAX_PATH);

	s_strcpy (romaji_path, config.data_path, MAX_PATH);
	s_strcat (romaji_path, romaji_dir, MAX_PATH);
}

/*--------------------------------------------------------------------*/

void test_main (gchar *text)
{
gchar answer[8];	// 5 is enough :)
gint number, i;

	number = qst_hash[q_counter];
	number %= N_SIGNS;

	s_strcpy (answer, jsigns[number], 8);
	
	for(i=0;i<8;i++)
		if(answer[i]=='_') answer[i]='\0';


	if (strcmp(text, answer)) {

		gtk_widget_set_sensitive (rm_entry, FALSE);
		gtk_widget_set_sensitive (stop_button, FALSE);

		gui_put_sign (qst_hash[q_counter], ROMAJI);

		while (gtk_events_pending())
		    gtk_main_iteration();

		sleep (1);

		gtk_widget_set_sensitive (rm_entry, TRUE);
		gtk_widget_set_sensitive (stop_button, TRUE);

		gtk_widget_grab_focus (rm_entry);

		a_counter++;
	}

	q_counter++;

	gui_set_progress();
	gtk_entry_set_text (GTK_ENTRY(rm_entry), "");

	if(q_counter == max_entries) {

		tval = ((difftime (time(NULL), timer_int_l)) / 60.0);

		config.t_time += tval;

		switch (test_mode) {
			case HIRAGANA:
				config.hiragana_counter++;
				if (config.hiragana_max_result < (q_counter-a_counter))
					config.hiragana_max_result = q_counter-a_counter;
				config.hiragana_time += tval;
				break;
			case KATAKANA:
				config.katakana_counter++;
				if (config.katakana_max_result < (q_counter-a_counter))
					config.katakana_max_result = q_counter-a_counter;
				config.katakana_time += tval;
				break;
			case MIXED:
				config.mixed_counter++;
				if (config.mixed_max_result < (q_counter-a_counter))
					config.mixed_max_result = q_counter-a_counter;
				config.mixed_time += tval;
				break;
		}

		gui_show_stat (test_mode);
		gui_disable_test();

	} else
		gui_put_sign (qst_hash[q_counter], test_mode);
}

/*--------------------------------------------------------------------*/


gchar* minutes2str (gfloat m)
{
static gchar obuffer[512];
guint d, h;

	d = m / (24 * 60);
	m = m - d * (24 * 60);
	h = m / 60;
	m = m - h * 60;
    
	sprintf (obuffer, "%2d days, %2d hours, %.2f minutes", d, h, m);
	return obuffer;
}

/*--------------------------------------------------------------------*/

